import sys
from scitbx.array_family import flex
import iotbx.pdb
from cctbx import miller
import mmtbx.utils
from libtbx.utils import null_out
from mmtbx import map_tools
from mmtbx.maps import fem
    
def run(args):
  inputs = mmtbx.utils.process_command_line_args(args = args)
  df = mmtbx.utils.determine_data_and_flags(
    reflection_file_server  = inputs.get_reflection_file_server(),
    log = null_out())
  pdb_inp = iotbx.pdb.input(file_name=inputs.pdb_file_names[0])
  xrs = pdb_inp.xray_structure_simple()
  fmodel = mmtbx.f_model.manager(
    xray_structure = xrs,
    f_obs          = df.f_obs,
    r_free_flags   = df.r_free_flags)
  fmodel.update_all_scales()
  print "r_work=%6.4f r_free=%6.4f"%(fmodel.r_work(), fmodel.r_free())
  fmodel.show()
  #
  crystal_gridding = fmodel.f_obs().crystal_gridding(
    d_min              = fmodel.f_obs().d_min(),
    resolution_factor  = 0.25)
  #
  mc_orig = map_tools.electron_density_map(
    fmodel=fmodel.deep_copy()).map_coefficients(
      map_type     = "2mFo-DFc",
      isotropize   = True,
      fill_missing = False)
  mc_fill_resolve = map_tools.electron_density_map(
    fmodel=fmodel.deep_copy()).map_coefficients(
      map_type            = "2mFo-DFc",
      fill_missing_method = "resolve_dm",
      isotropize   = True,
      fill_missing = True)
  mc_fill_f_model = map_tools.electron_density_map(
    fmodel=fmodel.deep_copy()).map_coefficients(
      map_type            = "2mFo-DFc",
      fill_missing_method = "f_model",
      isotropize   = True,
      fill_missing = True)
  #
  mtz_dataset = mc_orig.as_mtz_dataset(column_root_label="orig")
  mtz_dataset.add_miller_array(
    miller_array=mc_fill_resolve,
    column_root_label="resolve")
  mtz_dataset.add_miller_array(
    miller_array=mc_fill_f_model,
    column_root_label="f_model")
  mtz_object = mtz_dataset.mtz_object()
  mtz_object.write(file_name = "map_coeffs.mtz")
  #
  fem.ccp4_map(cg=crystal_gridding, file_name="orig.ccp4", mc=mc_orig)
  fem.ccp4_map(cg=crystal_gridding, file_name="resolve.ccp4", mc=mc_fill_resolve)
  fem.ccp4_map(cg=crystal_gridding, file_name="f_model.ccp4", mc=mc_fill_f_model)
  
if (__name__ == "__main__"):
  run(sys.argv[1:])

